<?php

namespace app\admin\controller;
use app\common\model\zdjr\Project;
use app\common\model\zdjr\ProjectBot;
use think\facade\Db;

class Zdjrproject extends Botbase
{
    /**
     * @var Project
     */
    protected $model;
    /**
     * @var ProjectBot
     */
    private $projectBotM;

    public function initialize()
    {
        parent::initialize(); // TODO: Change the autogenerated stub
        $this->model = new Project();
        $this->projectBotM = new ProjectBot();
    }

    /**
     * 列表
     * @return mixed
     * @throws \think\Exception
     * @throws \think\db\exception\DataNotFoundException
     * @throws \think\db\exception\ModelNotFoundException
     * @author: fudaoji<fdj@kuryun.cn>
     */
    public function index(){
        if (request()->isPost()) {
            $post_data = input('post.');
            $where = ['admin_id' => $this->adminInfo['id']];
            !empty($post_data['search_key']) && $where['title'] = ['like', '%' . $post_data['search_key'] . '%'];

            $total = $this->model->total($where, true);
            if ($total) {
                $list = $this->model->getList(
                    [$post_data['page'], $post_data['limit']], $where,
                    ['id' => 'asc'], true, true
                );
            } else {
                $list = [];
            }
            $this->success('success', '', ['total' => $total, 'list' => $list]);
        }

        $builder = new ListBuilder();
        $builder->addTopButton('addnew')
            ->setSearch([
                ['type' => 'text', 'name' => 'search_key', 'title' => '关键词', 'placeholder' => '名称|账号']
            ])
            ->addTableColumn(['title' => '名称', 'field' => 'title', 'minWidth' => 100])
            ->addTableColumn(['title' => '创建时间', 'field' => 'create_time', 'minWidth' => 120,'type' => 'datetime'])
            ->addTableColumn(['title' => '状态', 'field' => 'status', 'minWidth' => 80,'type' => 'enum','options' => [0 => '禁用', 1=> '启用']])
            ->addTableColumn(['title' => '操作', 'minWidth' => 200, 'type' => 'toolbar'])
            ->addRightButton('edit')
            ->addRightButton('edit', ['title' => '绑定机器人', 'href' => url('bind', ['project_id' => '__data_id__'])])
            ->addRightButton('self',['title' => '禁用/启用', 'href' => url('forbidPost', ['id' => '__data_id__']), 'data-ajax' => 1, 'data-confirm' => 1]);
        return $builder->show();
    }

    /**
     * 编辑
     * @return mixed
     * @throws \think\Exception
     * Author: fudaoji<fdj@kuryun.cn>
     */
    public function edit(){
        $id = input('id', null);
        $data = $this->model->getOneByMap(['id' => $id, 'admin_id' => $this->adminInfo['id']], true, true);

        if (!$data) {
            $this->error('参数错误');
        }

        $builder = new FormBuilder();
        $builder->setPostUrl(url('savePost'))
            ->addFormItem('id', 'hidden', 'id', 'id')
            ->addFormItem('title', 'text', '名称', '50字内', [], 'required maxlength=50')
            ->addFormItem('status', 'radio', '状态', '状态', [1 => '启用', 0 => '禁用'], 'required')
            ->setFormData($data);

        return $builder->show();
    }

    /**
     * 新增
     * @return mixed
     * @throws \think\Exception
     * Author: fudaoji<fdj@kuryun.cn>
     */
    public function add(){
        $builder = new FormBuilder();
        $builder->setPostUrl(url('savePost'))
            ->addFormItem('title', 'text', '名称', '50字内', [], 'required maxlength=50');

        return $builder->show();
    }

    /**
     * 禁用/启用
     * @throws \think\Exception
     * Author: fudaoji<fdj@kuryun.cn>
     */
    public function forbidPost()
    {
        $id = input('id', null);
        $data = $this->model->getOneByMap(['id' => $id, 'admin_id' => $this->adminInfo['id']], true, true);

        if (!$data) {
            $this->error('参数错误');
        }
        $this->model->updateOne([
            'id' => $id,
            'status' => abs($data['status'] - 1)
        ]);
        $this->success('操作成功');
    }

    public function bind(){
        if(request()->isPost()){
            $post_data = input('post.');
            $bots = explode(',', $post_data['bots']);
            if(!count($bots)){
                $this->error('请选择机器人');
            }
            foreach ($bots as $bot_id){
                if(! $this->projectBotM->total(['admin_id' => $this->adminInfo['id'],'bot_id' => $bot_id, 'project_id' => $post_data['project_id']])){
                    $this->projectBotM->addOne(['admin_id' => $this->adminInfo['id'],'bot_id' => $bot_id, 'project_id' => $post_data['project_id']]);
                }
            }
            $this->projectBotM->delByMap(['project_id' => $post_data['project_id'], 'bot_id' => ['notin', $bots]]);
            $this->success('操作成功!', '/undefined');
        }
        $project_id = input('project_id', 0);
        if(! $data = $this->model->getOne($project_id)){
            $this->error('数据不存在');
        }
        $data['bots'] = $this->projectBotM->getField('bot_id', ['admin_id' => $this->adminInfo['id'], 'project_id' => $project_id]);
        $exist = $this->projectBotM->getField('bot_id', ['admin_id' => $this->adminInfo['id'], 'project_id' => ['<>', $project_id]]);
        $bots = $this->getBots(['id' => ['notin', count($exist) ? $exist : [0]]]);
        $builder = new FormBuilder();
        $builder->setPostUrl(url('bind'))
            ->addFormItem('project_id', 'hidden', 'id', 'id')
            ->addFormItem('title', 'static', '项目', $data['title'])
            ->addFormItem('bots', 'chosen_multi', '选择机器人', '选择机器人', $bots, 'required')
            ->setFormData($data);

        return $builder->show();
    }
}